////////////////////////////////////////////////////////////////////////////////
//                            SFMatrix34f.h                                   //
////////////////////////////////////////////////////////////////////////////////

#ifndef MATRIX34F_H
#define MATRIX34F_H

#include "SFVec3f.h"
#include "SFPoint3f.h"

#include <iostream>
#include <vector>

namespace X3DTK {

/// class providing an implementation of the matrices in homogeneous coordinates.
/// The matrices are represented by 3*4 elements to economize memory and unnecessary
/// calculus. The fourth coordinates is not represented because the first three 
/// columns represent the vectors and the fourth the point.

class SFMatrix34f
{
public:
  /// 1, 1 coordinate
  float _11;
  /// 2, 1 coordinate
  float _21;
  /// 3, 1 coordinate
  float _31;

  /// 1, 2 coordinate		
  float _12;
  /// 2, 2 coordinate
  float _22;
  /// 3, 2 coordinate
  float _32;

  /// 1, 3 coordinate
  float _13;
  /// 2, 3 coordinate
  float _23;
  /// 3, 3 coordinate
  float _33;

  /// 1, 4 coordinate
  float _14;
  /// 2, 4 coordinate
  float _24;
  /// 3, 4 coordinate
  float _34;

  /// default constructor
  SFMatrix34f();
  /// constructor with the coordinates in parameters
  SFMatrix34f(float _11, float _21, float _31, float _12, float _22, float _32, float _13, float _23, float _33, float _14, float _24, float _34);
  /// constructor from a GL matrix
  explicit SFMatrix34f(const float GLMatrix[16]);
  /// constructor from the three vectors and the point
  SFMatrix34f(const SFVec3f &vx, const SFVec3f &vy, const SFVec3f &vz, const SFPoint3f &p);
  /// copy constructor
  SFMatrix34f(const SFMatrix34f &m);

  /// overloaded operator+
  friend SFMatrix34f operator+ (const SFMatrix34f &m1, const SFMatrix34f &m2);
  /// overloaded operator-
  friend SFMatrix34f operator- (const SFMatrix34f &m1, const SFMatrix34f &m2);
  /// overloaded operator*
  friend SFMatrix34f operator* (const SFMatrix34f &m1, const SFMatrix34f &m2);
  /// overloaded operator*
  friend SFMatrix34f operator* (const float a, const SFMatrix34f &m);
  /// overloaded operator*
  friend SFVec3f operator* (const SFMatrix34f &m, const SFVec3f &v);
  /// overloaded operator*
  friend SFPoint3f operator* (const SFMatrix34f &m, const SFPoint3f &v);
  /// general inverse of the matrix
  SFMatrix34f i();
  /// inverse of the matrix taking in account that the matrix is unit and performing
  /// the operation faster than i() 
  SFMatrix34f iu();
  /// conversion to float[16]
  void toFloat16(std::vector<float> &M) const;
};

/// returns the identity matrix
SFMatrix34f identity34();
/// returns a scale matrix
SFMatrix34f scale34(float sx, float sy, float sz);
/// returns a rotation matrix around the x axis
SFMatrix34f rotationX(float angle);
/// returns a rotation matrix around the y axis
SFMatrix34f rotationY(float angle);
/// returns a rotation matrix around the z axis
SFMatrix34f rotationZ(float angle);
/// returns a translation matrix
SFMatrix34f translation(const SFVec3f &t);
/// returns a general rotation matrix
SFMatrix34f rotation(float angle, const SFVec3f &u); 
/// overloaded operator<< for SFMatrix34f
std::ostream& operator<<(std::ostream& o, const SFMatrix34f &M);

}

#endif

